/* PS_SOCK.H */
#ifndef _PS_SOCK_H
#define _PS_SOCK_H

#include<stdio.h>
#include<stdlib.h>

#if defined (NT)
#include <windows.h>
#include <winsock.h>
#include <wincrypt.h>
#else
#define SOCKET int
#include <sys/time.h>
#endif

#ifdef USE_SSLEAY
#include <openssl/err.h>
#include <openssl/x509.h>
#include <openssl/ssl.h>
#include <openssl/rand.h>
#include <openssl/pem.h>

#ifdef NT
#undef X509_NAME /*undef the X509_NAME macro from wincrypt.h, use the typedef from openssl/x509.h*/
#endif
#endif

/* Used as a bitmap, except for "certificate or password"
   which is denoted by 0 */
enum { 
 AUTH_REQ_PASSWORD=1,
 AUTH_REQ_CERT=2
};

#include "header.h"


#define PS_SERVERCERT  "servercert.pem"
#define PS_SERVERKEY   "../private/serverkey.pem"
#define PS_SERVERTRUSTEDCERTS "trusted.pem"

#define PS_CLIENTCERT  "certs/servercert.pem"
#define PS_CLIENTKEY   "private/serverkey.pem"
#define PS_CLIENTTRUSTEDCERTS "certs/trusted.pem"

#ifdef USE_SSLEAY
typedef SSL_CTX *PS_SSL_CTX;
#else
typedef void *PS_SSL_CTX;
#endif

typedef struct PS_SOCKET_STRUCT *PS_SOCKET;
typedef struct PS_CTX_STRUCT *PS_CTX;

#define MAX_CN_SZ 65
#define MAX_OU_SZ 129
#define MAX_O_SZ 65
#define MAX_L_SZ 129
#define MAX_ST_SZ 129
#define MAX_C_SZ 3
#define MAX_EMAIL_SZ 129


#ifdef NO_THREADS
# define PS_SOCK_LOCKS 0
#else
# define PS_SOCK_LOCKS CRYPTO_NUM_LOCKS
typedef void (*locking_callback)(int mode, int type,const char *file, int line);
void ps_sock_set_locking_callback(locking_callback);
#endif


#ifdef USE_SSLEAY
PS_SSL_CTX PS_CTX_getSSL_CTX(PS_CTX ps_ctx);
bool PS_CTX_getIsServer(PS_CTX ps_ctx);
char * PS_CTX_getServerKey(PS_CTX ps_ctx);
char * PS_CTX_getServerCert(PS_CTX ps_ctx); 
char * PS_CTX_getCAs(PS_CTX ps_ctx);
#endif

void PS_init(); /*automatically called by PS_newCtx()*/

PS_CTX PS_newCtx(bool isServer, const char *tlsCipherSuites, const char *own_cert_file, const char *own_key_file, const char *trusted_certs_file);
PS_CTX PS_newSecureServerCtx(const char *sslroot, const char *tlsCipherSuits, bool authentication);
PS_CTX PS_newSecureClientCtx(const char *sslroot, const char *tlsCipherSuits, bool authentication);
void PS_freeCtx(PS_CTX ctx);
PS_SOCKET PS_new();
char *PS_getPeerCN(PS_SOCKET so, char *cn); /*cn must have enought space for a CN*/
char *PS_getPeerEMail(PS_SOCKET so, char *email); /*cn must have enought space for an EMail*/

SOCKET PS_getPureSocket(PS_SOCKET ps_so);
bool PS_handshake(PS_SOCKET ps_so, SOCKET so, PS_CTX ctx);
bool PS_isSecure(PS_SOCKET ps_so);
bool PS_recv_line(PS_SOCKET ps_so, char *buf, int len);
int PS_recv_all(PS_SOCKET ps_so, char *buf, int len);
int PS_recv_all_timeout(PS_SOCKET ps_so, char *buf, int len, int secs, int millisecs);
int PS_send_all(PS_SOCKET ps_so, const char *buf, int len);
int PS_printf(PS_SOCKET ps_so, const char *fmt, ...);
bool PS_send_file(PS_SOCKET ps_so, const char *fname, char *buffer, int buffsize);
bool PS_send_file_range(PS_SOCKET so, const char *fname, char *buffer, uint32 buffsize, uint32 offset_s, uint32 offset_e);

void PS_resetSentBytesCounter(PS_SOCKET ps_so);
bool PS_getSentBytesCounter(PS_SOCKET ps_so, uint32 *p);

int  PS_closesocket(PS_SOCKET ps_so);
int PS_shutdown(PS_SOCKET pps_so, int mode);
void PS_release(PS_SOCKET so);


bool PS_authenticateServerAddress(PS_SOCKET ps_so, const char server[]);
bool PS_matchesHost(const char host[], const char certCN[]);  


#endif
