#include "chkalloc.h"
#include "readdir.h"
#include "header.h"

#ifdef NT /****************NT************************/
#include <string.h>
#include <windows.h>
#include <errno.h>

/**********************************************************************
 * Implement dirent-style opendir/readdir/closedir on Window 95/NT
 *
 * Functions defined are opendir(), readdir() and closedir() with the
 * same prototypes as the normal dirent.h implementation.
 *
 * The DIR typedef is not compatible with Unix.
 **********************************************************************/

DIR *opendir(const char *dir){
	DIR *dp;
	char *filespec;
	long handle;
	int index;
	filespec=checked_malloc(strlen(dir)+2+1);
	strcpy(filespec,dir);
	index=strlen(filespec)-1;
    if(index>=0 && (filespec[index]=='/' || filespec[index]=='\\')){
		filespec[index]='\0';
    }
	strcat(filespec,"/*");
	dp=(DIR *)checked_malloc(sizeof(DIR));
	dp->offset=0;
	dp->finished=0;
	dp->dir=checked_strdup(dir);
	if((handle=_findfirst(filespec,&(dp->fileinfo)))<0){
        if (errno == ENOENT){
			dp->finished = 1;
        }
        else{
			return NULL;
        }
	}
	dp->handle=handle;
	checked_free(filespec);
	return dp;
}

struct dirent *readdir(DIR * dp){
    if (!dp || dp->finished){
		return NULL;
    }
	if(dp->offset!=0){
		if(_findnext(dp->handle,&(dp->fileinfo))<0){
			dp->finished=1;
			return NULL;
		}
	}
	dp->offset++;
	strncpy(dp->dent.d_name,dp->fileinfo.name,_MAX_FNAME);
	dp->dent.d_ino=1;
	dp->dent.d_reclen=strlen(dp->dent.d_name);
	dp->dent.d_off=dp->offset;
	return &(dp->dent);
}

int closedir(DIR * dp){
    if(!dp){
		return 0;
    }
	_findclose(dp->handle);
    if(dp->dir){
		checked_free(dp->dir);
    }
    if(dp){
		checked_free(dp);
    }
	return 0;
}

void rewinddir(DIR *dir_Info){
	char *filespec;
	long handle;
	int index;
	dir_Info->handle = 0;
	dir_Info->offset = 0;
	dir_Info->finished = 0;
	filespec=checked_malloc(strlen(dir_Info->dir)+2+1);
	strcpy(filespec,dir_Info->dir);
	index=strlen(filespec)-1;
    if(index>=0 && (filespec[index]=='/' || filespec[index]=='\\')){
		filespec[index] = '\0';
    }
	strcat(filespec, "/*");
	if((handle=_findfirst(filespec,&(dir_Info->fileinfo)))<0){
		if(errno == ENOENT){
			dir_Info->finished=1;
		}
	}
	dir_Info->handle=handle;
	checked_free(filespec);
}

int copyfile(const char *from,const char *to){
    return (CopyFile(from,to,false)==0?-1:0);
}

#else /****************Unix**************************/
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>

#define BF_SZ (64*1024)

int copyfile(const char *from,const char *to){
    char bf[BF_SZ];
    uint32 count=0;
    int fdfrom,n,fdto=open(to,O_CREAT|O_TRUNC|O_WRONLY,S_IWRITE|S_IREAD);
    if(fdto<0){
        return -1;
    }
    fdfrom=open(from,O_RDONLY,S_IREAD);
    if(fdfrom<0){
        close(fdto);
        return -1;
    }
    while((n=read(fdfrom,bf,BF_SZ))>0){
        count+=n;
        if(write(fdto,bf,n)<n){
            close(fdto);
            close(fdfrom);
            return -1;
        }
    }
    if(close(fdto)<0){
        return -1;
    }
    if(close(fdfrom)<0){
        return -1;
    }
    return count;
}

#endif
